# -*- coding:utf-8 -*-
from django.forms import Widget, Select


def get_all_fields_from_form(instance):
    fields = list(instance().base_fields)

    for field in list(instance().declared_fields):
        if field not in fields:
            fields.append(field)
    return fields


class SelectSearch(Select):
    template_name = "ajaxlte/widgets/select-search.html"

    def build_attrs(self, base_attrs, extra_attrs=None):
        extra_attrs.update({
            'class': "selectpicker",
            'data-live-search': "true",
        })
        return super().build_attrs(base_attrs, extra_attrs=extra_attrs)


class TableBordered(Widget):
    template_name = "ajaxlte/widgets/table-bordered.html"

    def format_value(self, value):
        return value


class TableBorderedInput(Widget):
    template_name = "ajaxlte/widgets/table-bordered-input.html"

    def format_value(self, value):
        if value:
            return [self.attrs['form'](instance=x) for x in value]
        return value

    def build_attrs(self, base_attrs, extra_attrs=None):
        updated_attrs = dict()
        updated_attrs['columns'] = get_all_fields_from_form(base_attrs['form'])
        updated_attrs['model'] = str(base_attrs['form']._meta.model.__name__).lower() + "_id"
        updated_attrs['opts'] = base_attrs['form']._meta.model._meta
        extra_attrs.update(updated_attrs)
        return super().build_attrs(base_attrs, extra_attrs)


class FormWidget(Widget):
    template_name = "ajaxlte/widgets/form-widget.html"

    def format_value(self, value):
        try:
            return self.attrs['form'](instance=self.attrs['model'].objects.get(id=value))
        except:
            return self.attrs['form']


# -*- coding:utf-8 -*-
import json
from django.http import HttpResponseBadRequest
from django.utils.deprecation import MiddlewareMixin


class PutParsingMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if request.method == "PUT" and request.content_type != "application/json":
            if hasattr(request, '_post'):
                del request._post
                del request._files
            try:
                request.method = "POST"
                request._load_post_and_files()
                request.method = "PUT"
            except AttributeError:
                request.META['REQUEST_METHOD'] = 'POST'
                request._load_post_and_files()
                request.META['REQUEST_METHOD'] = 'PUT'

            request.PUT = request.POST


class DeleteParsingMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if request.method == "DELETE" and request.content_type != "application/json":
            if hasattr(request, '_post'):
                del request._post
                del request._files
            try:
                request.method = "POST"
                request._load_post_and_files()
                request.method = "PUT"
            except AttributeError as e:
                request.META['REQUEST_METHOD'] = 'POST'
                request._load_post_and_files()
                request.META['REQUEST_METHOD'] = 'DELETE'

            request.DELETE = request.POST


class JSONParsingMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if (request.method == "PUT" or request.method == "POST") and request.content_type == "application/json":
            try:
                request.JSON = json.loads(request.body)
            except ValueError as ve:
                return HttpResponseBadRequest("unable to parse JSON data. Error : {0}".format(ve))


from django import template
import re
from django.urls import reverse
from django.template.loader import render_to_string


class _Menu:
    parents = []
    children = []
    models_icon = {}

    def clear(self):
        self.parents = []
        self.children = []

    def add(self, label, link='', icon='', id_='', parent=''):

        if id_ == '':
            id_ = label

        if parent != '':
            child = {
                id_: {
                    'label': label,
                    'link': link,
                    'icon': icon,
                    'children': []
                }
            }

            self.children.append(child)

            for idx, parent_item in enumerate(self.parents):

                if parent in parent_item:
                    self.parents[idx][parent]['children'].append(child)
                else:
                    for idx, child_item in enumerate(self.children):
                        if parent in child_item:
                            self.children[idx][parent]['children'].append(child)

        else:
            self.parents.append({
                id_: {
                    'label': label,
                    'link': link,
                    'icon': icon,
                    'children': []
                }
            })

    def render(self, context, menus=None):
        context['menu'] = menus
        return render_to_string('adminlte/menu.html', context=context)

    def admin_apps(self, context, r):
        request = context['request']
        for app in context['available_apps']:
            if str(app['app_url']) in request.path:
                r += '<li class="nav-item has-treeview menu-open"><a href="#" class="nav-link active"><i class="nav-icon fas fa-edit"></i> <p>%s</p><p><i class="fas fa-angle-left right"></i></p></a><ul class="nav nav-treeview">\n' % (
                    app['name'])
            else:
                r += '<li class="nav-item has-treeview"><a href="#" class="nav-link"><i class="nav-icon fas fa-edit"></i> <p>%s</p><p><i class="fas fa-angle-left right"></i></p></a><ul class="nav nav-treeview">\n' % (
                    app['name'])

            for model in app['models']:
                if 'add_url' in model:
                    url = model['add_url']

                if 'change_url' in model:
                    url = model['change_url']

                # if 'delete_url' in model:
                #     url = model['delete_url']

                if 'admin_url' in model:
                    url = model['admin_url']

                icon = '<i class="far fa-circle nav-icon"></i>'
                if model['object_name'].title() in self.models_icon:
                    if self.models_icon[model['object_name'].title()] != '':
                        if re.match(r'\<([a-z]*)\b[^\>]*\>(.*?)\<\/\1\>',
                                    self.models_icon[model['object_name'].title()]):
                            icon = self.models_icon[model['object_name'].title()]
                        else:
                            icon = '<i class="%s"></i>' % (self.models_icon[model['object_name'].title()])
                if request.path == url:
                    r += '<li class="nav-item"><a href="%s" class="nav-link active">%s %s</a></li>' % (
                        url, icon, model['name'])
                else:
                    r += '<li class="nav-item"><a href="%s" class="nav-link">%s %s</a></li>' % (
                        url, icon, model['name'])

            r += '</ul></li>\n'

        return r

    def app_menu(self, context, r):
        return r

    def set_model_icon(self, model_name, icon):
        self.models_icon[model_name.title()] = icon

    def get_model_icon(self, context):

        icon = '<i class="far fa-circle nav-icon"></i>'
        if context['model']['object_name'].title() in self.models_icon:

            if self.models_icon[context['model']['object_name'].title()] != '':
                if re.match(r'<([a-z]*)\b[^>]*>(.*?)</\1>', self.models_icon[context['model']['object_name'].title()]):
                    icon = self.models_icon[context['model']['object_name']]
                else:
                    icon = '<i class="%s"></i>' % (self.models_icon[context['model']['object_name'].title()])

        return icon


register = template.Library()

Menu = _Menu()


@register.simple_tag(takes_context=True, name='main_menu')
def menu_tag(context):
    return Menu.admin_apps(context, Menu.render(context))


@register.simple_tag(takes_context=True, name='icon')
def icon_tag(context):
    return Menu.get_model_icon(context)



from django.urls import path
from .ajax import *

app_name = "ajaxlte"

urlpatterns = [
    path('get_object/', get_object, name="getObject"),
    path('object_update/', object_update, name="ObjectUpdate"),
    path('object_view/', object_view, name="ObjectView"),
    path('get_collection/', get_collection, name="getCollection"),
    path('get_datatables/', get_datatables, name="getDataTables"),
    path('autocomplete/', autocomplete, name="autocomplete"),
    path('object_execute/', object_execute, name="objectExecute"),
    path('get_html_form/', get_html_form, name="getHtmlForm"),
]



# -*- coding:utf-8 -*-
from django.db.models import Model
from django.forms.models import model_to_dict


class BaseModel(Model):

    def __iter__(self):
        for field_name in self._meta.fields:
            try:
                value = getattr(self, field_name)
            except:
                value = None
            yield (field_name, value)

    def __getitem__(self, fieldname):
        try:
            return getattr(self, fieldname)
        except:
            return None

    def __setitem__(self, fieldname, value):
        try:
            return setattr(self, fieldname, value)
        except:
            return None

    def __str__(self):
        if self.pk:
            return self._meta.verbose_name + "(%s)" % str(self.pk)
        else:
            return self._meta.verbose_name

    def to_json(self):
        o = model_to_dict(self)
        o['str'] = self.__str__()
        o['app_label'] = self._meta.app_label
        o['model'] = self._meta.object_name.lower()
        return o

    class Meta:
        abstract = True